@@ -13,7 +13,8 @@
 
 import mock
 import mox
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_utils import uuidutils
 import six
 
 from heat.common import exception
@@ -23,10 +24,9 @@ from heat.engine import parser
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.engine import template
-from heat.openstack.common import uuidutils
 from heat.tests import common
+from heat.tests.nova import fakes
 from heat.tests import utils
-from heat.tests.v1_1 import fakes
 
 from ..resources import cloud_server  # noqa
 
@@ -44,7 +44,7 @@ wp_template = '''
   },
   "Resources" : {
     "WebServer": {
-      "Type": "Rackspace::Cloud::Server",
+      "Type": "OS::Nova::Server",
       "Properties": {
         "image" : "CentOS 5.2",
         "flavor"   : "256 MB Server",
@@ -71,11 +71,10 @@ class CloudServersTest(common.HeatTestCase):
             'nova')._create = mock_nova_create
         mock_nova_create.return_value = self.fc
 
-        self.stub_keystoneclient()
         # Test environment may not have pyrax client library installed and if
         # pyrax is not installed resource class would not be registered.
         # So register resource provider class explicitly for unit testing.
-        resource._register_class("Rackspace::Cloud::Server",
+        resource._register_class("OS::Nova::Server",
                                  cloud_server.CloudServer)
 
     def _mock_get_image_id_success(self, imageId):
@@ -90,9 +89,10 @@ class CloudServersTest(common.HeatTestCase):
 
     def _setup_test_stack(self, stack_name):
         t = template_format.parse(wp_template)
-        templ = template.Template(t)
+        templ = template.Template(
+            t, env=environment.Environment({'key_name': 'test'}))
+
         stack = parser.Stack(self.ctx, stack_name, templ,
-                             environment.Environment({'key_name': 'test'}),
                              stack_id=uuidutils.generate_uuid())
         return (templ, stack)
 
@@ -101,15 +101,15 @@ class CloudServersTest(common.HeatTestCase):
         stack_name = '%s_s' % name
         (tmpl, stack) = self._setup_test_stack(stack_name)
 
-        tmpl.t['Resources']['WebServer']['Properties']['image'] = \
-            image_id or 'CentOS 5.2'
-        tmpl.t['Resources']['WebServer']['Properties']['flavor'] = \
-            '256 MB Server'
+        tmpl.t['Resources']['WebServer']['Properties'][
+            'image'] = image_id or 'CentOS 5.2'
+        tmpl.t['Resources']['WebServer']['Properties'][
+            'flavor'] = '256 MB Server'
 
         server_name = '%s' % name
         if override_name:
-            tmpl.t['Resources']['WebServer']['Properties']['name'] = \
-                server_name
+            tmpl.t['Resources']['WebServer']['Properties'][
+                'name'] = server_name
 
         resource_defns = tmpl.resource_definitions(stack)
         server = cloud_server.CloudServer(server_name,
@@ -132,6 +132,7 @@ class CloudServersTest(common.HeatTestCase):
                 nics=None,
                 availability_zone=None,
                 block_device_mapping=None,
+                block_device_mapping_v2=None,
                 config_drive=True,
                 disk_config=None,
                 reservation_id=None,
@@ -325,111 +326,6 @@ class CloudServersTest(common.HeatTestCase):
         self.assertEqual('Error: Unknown Managed Cloud automation status: FOO',
                          six.text_type(exc))
 
-    @mock.patch.object(resource.Resource, 'data_set')
-    def test_create_store_admin_pass_resource_data(self,
-                                                   mock_data_set):
-        self._mock_metadata_os_distro()
-        return_server = self.fc.servers.list()[1]
-        return_server.adminPass = 'autogenerated'
-        stack_name = 'admin_pass_s'
-        (t, stack) = self._setup_test_stack(stack_name)
-
-        t.t['Resources']['WebServer']['Properties']['save_admin_pass'] = True
-        resource_defns = t.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-
-        self.fc.servers.create = mock.Mock(return_value=return_server)
-        self._mock_get_image_id_success('image_id')
-        scheduler.TaskRunner(server.create)()
-        expected_call = mock.call(server.ADMIN_PASS,
-                                  'autogenerated', redact=True)
-        self.assertIn(expected_call, mock_data_set.call_args_list)
-
-    @mock.patch.object(resource.Resource, 'data_set')
-    def test_create_save_admin_pass_is_false(self, mock_data_set):
-        self._mock_metadata_os_distro()
-        return_server = self.fc.servers.list()[1]
-        return_server.adminPass = 'autogenerated'
-        stack_name = 'admin_pass_s'
-        (t, stack) = self._setup_test_stack(stack_name)
-
-        t.t['Resources']['WebServer']['Properties']['save_admin_pass'] = False
-        resource_defns = t.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-
-        self.fc.servers.create = mock.Mock(return_value=return_server)
-        self._mock_get_image_id_success('image_id')
-        scheduler.TaskRunner(server.create)()
-        expected_call = mock.call(mock.ANY, server.ADMIN_PASS,
-                                  mock.ANY, mock.ANY)
-        self.assertNotIn(expected_call, mock_data_set.call_args_list)
-
-    @mock.patch.object(resource.Resource, 'data_set')
-    def test_create_save_admin_pass_defaults_to_false(self,
-                                                      mock_data_set):
-        self._mock_metadata_os_distro()
-        return_server = self.fc.servers.list()[1]
-        return_server.adminPass = 'autogenerated'
-        stack_name = 'admin_pass_s'
-        (t, stack) = self._setup_test_stack(stack_name)
-
-        t.t['Resources']['WebServer']['Properties']['save_admin_pass'] = None
-        resource_defns = t.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-
-        self.fc.servers.create = mock.Mock(return_value=return_server)
-        self._mock_get_image_id_success('image_id')
-
-        scheduler.TaskRunner(server.create)()
-        expected_call = mock.call(mock.ANY, server.ADMIN_PASS,
-                                  mock.ANY, mock.ANY)
-        self.assertNotIn(expected_call, mock_data_set.call_args_list)
-
-    @mock.patch.object(resource.Resource, 'data_set')
-    def test_create_without_adminPass_attribute(self,
-                                                mock_data_set):
-        self._mock_metadata_os_distro()
-        return_server = self.fc.servers.list()[1]
-        stack_name = 'admin_pass_s'
-        (tmpl, stack) = self._setup_test_stack(stack_name)
-
-        resource_defns = tmpl.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-
-        self.fc.servers.create = mock.Mock(return_value=return_server)
-        self._mock_get_image_id_success('image_id')
-
-        scheduler.TaskRunner(server.create)()
-        expected_call = mock.call(mock.ANY, server.ADMIN_PASS,
-                                  mock.ANY, redact=mock.ANY)
-        self.assertNotIn(expected_call, mock_data_set.call_args_list)
-
-    @mock.patch.object(resource.Resource, 'data')
-    def test_server_handles_server_without_password(self, mock_data_get):
-        mock_data_get.return_value = {}
-        stack_name = 'admin_pass_s'
-        (tmpl, stack) = self._setup_test_stack(stack_name)
-
-        resource_defns = tmpl.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-        self.assertEqual('', server.FnGetAtt('admin_pass'))
-
-    @mock.patch.object(resource.Resource, 'data')
-    def test_server_has_admin_pass_attribute_available(self, mock_data_get):
-        mock_data_get.return_value = {'admin_pass': 'foo'}
-        stack_name = 'admin_pass_s'
-        (tmpl, stack) = self._setup_test_stack(stack_name)
-
-        resource_defns = tmpl.resource_definitions(stack)
-        server = cloud_server.CloudServer('WebServer',
-                                          resource_defns['WebServer'], stack)
-        self.assertEqual('foo', server.FnGetAtt('admin_pass'))
-
     def _test_server_config_drive(self, user_data, config_drive, result):
         return_server = self.fc.servers.list()[1]
         stack_name = 'no_user_data'
@@ -457,6 +353,7 @@ class CloudServersTest(common.HeatTestCase):
             nics=mock.ANY,
             availability_zone=mock.ANY,
             block_device_mapping=mock.ANY,
+            block_device_mapping_v2=mock.ANY,
             config_drive=result,
             disk_config=mock.ANY,
             reservation_id=mock.ANY,
