@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,21 +11,48 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
+import copy
+import fnmatch
 import glob
 import itertools
 import os.path
+import warnings
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log
+from oslo_utils import encodeutils
+import six
 
-from heat.openstack.common import log
-from heat.openstack.common.gettextutils import _
-from heat.common import environment_format
+from heat.common import environment_format as env_fmt
 from heat.common import exception
-
+from heat.common.i18n import _
+from heat.common.i18n import _LE
+from heat.common.i18n import _LI
+from heat.common.i18n import _LW
+from heat.engine import support
 
 LOG = log.getLogger(__name__)
 
 
+HOOK_TYPES = (HOOK_PRE_CREATE, HOOK_PRE_UPDATE) = ('pre-create', 'pre-update')
+
+
+def valid_hook_type(hook):
+    return hook in HOOK_TYPES
+
+
+def is_hook_definition(key, value):
+    if key == 'hooks':
+        if isinstance(value, six.string_types):
+            return valid_hook_type(value)
+        elif isinstance(value, collections.Sequence):
+            return all(valid_hook_type(hook) for hook in value)
+        else:
+            return False
+    return False
+
+
 class ResourceInfo(object):
     """Base mapping of resource type to implementation."""
 
@@ -41,7 +67,7 @@ class ResourceInfo(object):
         if name.endswith(('.yaml', '.template')):
             # a template url for the resource "Type"
             return TemplateResourceInfo(registry, path, value)
-        elif not isinstance(value, basestring):
+        elif not isinstance(value, six.string_types):
             return ClassResourceInfo(registry, path, value)
         elif value.endswith(('.yaml', '.template')):
             # a registered template
@@ -101,8 +127,7 @@ class ClassResourceInfo(ResourceInfo):
 
 
 class TemplateResourceInfo(ResourceInfo):
-    """Store the info needed to start a TemplateResource.
-    """
+    """Store the info needed to start a TemplateResource."""
     description = 'Template'
 
     def __init__(self, registry, path, value):
@@ -115,12 +140,15 @@ class TemplateResourceInfo(ResourceInfo):
 
     def get_class(self):
         from heat.engine.resources import template_resource
+        env = self.registry.environment
         return template_resource.generate_class(str(self.name),
-                                                self.template_name)
+                                                self.template_name,
+                                                env)
 
 
 class MapResourceInfo(ResourceInfo):
     """Store the mapping of one resource type to another.
+
     like: OS::Networking::FloatingIp -> OS::Neutron::FloatingIp
     """
     description = 'Mapping'
@@ -134,6 +162,7 @@ class MapResourceInfo(ResourceInfo):
 
 class GlobResourceInfo(MapResourceInfo):
     """Store the mapping (with wild cards) of one resource type to another.
+
     like: OS::Networking::* -> OS::Neutron::*
     """
     description = 'Wildcard Mapping'
@@ -150,9 +179,10 @@ class GlobResourceInfo(MapResourceInfo):
 class ResourceRegistry(object):
     """By looking at the environment, find the resource implementation."""
 
-    def __init__(self, global_registry):
+    def __init__(self, global_registry, env):
         self._registry = {'resources': {}}
         self.global_registry = global_registry
+        self.environment = env
 
     def load(self, json_snippet):
         self._load_registry([], json_snippet)
@@ -165,12 +195,23 @@ class ResourceRegistry(object):
         for k, v in iter(registry.items()):
             if v is None:
                 self._register_info(path + [k], None)
+            elif is_hook_definition(k, v):
+                self._register_hook(path + [k], v)
             elif isinstance(v, dict):
                 self._load_registry(path + [k], v)
             else:
                 self._register_info(path + [k],
                                     ResourceInfo(self, path + [k], v))
 
+    def _register_hook(self, path, hook):
+        name = path[-1]
+        registry = self._registry
+        for key in path[:-1]:
+            if key not in registry:
+                registry[key] = {}
+            registry = registry[key]
+        registry[name] = hook
+
     def _register_info(self, path, info):
         """place the new info in the correct location in the registry.
         path: a list of keys ['resources', 'my_server', 'OS::Nova::Server']
@@ -188,15 +229,15 @@ class ResourceRegistry(object):
             if name.endswith('*'):
                 # delete all matching entries.
                 for res_name in registry.keys():
-                    if isinstance(registry[res_name], ResourceInfo) and \
-                       res_name.startswith(name[:-1]):
-                        LOG.warn(_('Removing %(item)s from %(path)s') % {
+                    if (isinstance(registry[res_name], ResourceInfo) and
+                            res_name.startswith(name[:-1])):
+                        LOG.warn(_LW('Removing %(item)s from %(path)s'), {
                             'item': res_name,
                             'path': descriptive_path})
                         del registry[res_name]
             else:
                 # delete this entry.
-                LOG.warn(_('Removing %(item)s from %(path)s') % {
+                LOG.warn(_LW('Removing %(item)s from %(path)s'), {
                     'item': name,
                     'path': descriptive_path})
                 registry.pop(name, None)
@@ -209,14 +250,78 @@ class ResourceRegistry(object):
                 'path': descriptive_path,
                 'was': str(registry[name].value),
                 'now': str(info.value)}
-            LOG.warn(_('Changing %(path)s from %(was)s to %(now)s') % details)
+            LOG.warn(_LW('Changing %(path)s from %(was)s to %(now)s'),
+                     details)
         else:
-            LOG.info(_('Registering %(path)s -> %(value)s') % {
+            LOG.info(_LI('Registering %(path)s -> %(value)s'), {
                 'path': descriptive_path,
                 'value': str(info.value)})
+
+        if isinstance(info, ClassResourceInfo):
+            if info.value.support_status.status != support.SUPPORTED:
+                warnings.warn(encodeutils.safe_encode(
+                    info.value.support_status.message))
+
         info.user_resource = (self.global_registry is not None)
         registry[name] = info
 
+    def remove_item(self, info):
+        if not isinstance(info, TemplateResourceInfo):
+            return
+
+        registry = self._registry
+        for key in info.path[:-1]:
+            registry = registry[key]
+        if info.path[-1] in registry:
+            registry.pop(info.path[-1])
+
+    def matches_hook(self, resource_name, hook):
+        '''Return whether a resource have a hook set in the environment.
+
+        For a given resource and a hook type, we check to see if the the passed
+        group of resources has the right hook associated with the name.
+
+        Hooks are set in this format via `resources`:
+
+            {
+                "res_name": {
+                    "hooks": [pre-create, pre-update]
+                },
+                "*_suffix": {
+                    "hooks": pre-create
+                },
+                "prefix_*": {
+                    "hooks": pre-update
+                }
+            }
+
+        A hook value is either `pre-create`, `pre-update` or a list of those
+        values. Resources support wildcard matching. The asterisk sign matches
+        everything.
+        '''
+        ress = self._registry['resources']
+        for name_pattern, resource in six.iteritems(ress):
+            if fnmatch.fnmatchcase(resource_name, name_pattern):
+                if 'hooks' in resource:
+                    hooks = resource['hooks']
+                    if isinstance(hooks, six.string_types):
+                        if hook == hooks:
+                            return True
+                    elif isinstance(hooks, collections.Sequence):
+                        if hook in hooks:
+                            return True
+        return False
+
+    def remove_resources_except(self, resource_name):
+        ress = self._registry['resources']
+        new_resources = {}
+        for name, res in six.iteritems(ress):
+            if fnmatch.fnmatchcase(resource_name, name):
+                new_resources.update(res)
+        if resource_name in ress:
+            new_resources.update(ress[resource_name])
+        self._registry['resources'] = new_resources
+
     def iterable_by(self, resource_type, resource_name=None):
         is_templ_type = resource_type.endswith(('.yaml', '.template'))
         if self.global_registry is not None and is_templ_type:
@@ -249,7 +354,7 @@ class ResourceRegistry(object):
                 yield self._registry[pattern]
 
     def get_resource_info(self, resource_type, resource_name=None,
-                          registry_type=None, accept_fn=None):
+                          registry_type=None):
         """Find possible matches to the resource type and name.
         chain the results from the global and user registry to find
         a match.
@@ -280,11 +385,10 @@ class ResourceRegistry(object):
         for info in sorted(matches):
             match = info.get_resource_info(resource_type,
                                            resource_name)
-            if ((registry_type is None or isinstance(match, registry_type)) and
-                    (accept_fn is None or accept_fn(info))):
+            if ((registry_type is None or isinstance(match, registry_type))):
                 return match
 
-    def get_class(self, resource_type, resource_name=None, accept_fn=None):
+    def get_class(self, resource_type, resource_name=None):
         if resource_type == "":
             msg = _('Resource "%s" has no type') % resource_name
             raise exception.StackValidationFailed(message=msg)
@@ -292,13 +396,12 @@ class ResourceRegistry(object):
             msg = _('Non-empty resource type is required '
                     'for resource "%s"') % resource_name
             raise exception.StackValidationFailed(message=msg)
-        elif not isinstance(resource_type, basestring):
+        elif not isinstance(resource_type, six.string_types):
             msg = _('Resource "%s" type is not a string') % resource_name
             raise exception.StackValidationFailed(message=msg)
 
         info = self.get_resource_info(resource_type,
-                                      resource_name=resource_name,
-                                      accept_fn=accept_fn)
+                                      resource_name=resource_name)
         if info is None:
             msg = _("Unknown resource Type : %s") % resource_type
             raise exception.StackValidationFailed(message=msg)
@@ -311,6 +414,8 @@ class ResourceRegistry(object):
             for k, v in iter(level.items()):
                 if isinstance(v, dict):
                     tmp[k] = _as_dict(v)
+                elif is_hook_definition(k, v):
+                    tmp[k] = v
                 elif v.user_resource:
                     tmp[k] = v.value
             return tmp
@@ -329,14 +434,10 @@ class ResourceRegistry(object):
                     cls.get_class().support_status.status ==
                     support_status.encode())
 
-        return [name for name, cls in self._registry.iteritems()
+        return [name for name, cls in six.iteritems(self._registry)
                 if is_resource(name) and status_matches(cls)]
 
 
-SECTIONS = (PARAMETERS, RESOURCE_REGISTRY) = \
-           ('parameters', 'resource_registry')
-
-
 class Environment(object):
 
     def __init__(self, env=None, user_env=True):
@@ -355,24 +456,47 @@ class Environment(object):
         else:
             global_registry = None
 
-        self.registry = ResourceRegistry(global_registry)
-        self.registry.load(env.get(RESOURCE_REGISTRY, {}))
+        self.registry = ResourceRegistry(global_registry, self)
+        self.registry.load(env.get(env_fmt.RESOURCE_REGISTRY, {}))
 
-        if 'parameters' in env:
-            self.params = env['parameters']
+        if env_fmt.PARAMETER_DEFAULTS in env:
+            self.param_defaults = env[env_fmt.PARAMETER_DEFAULTS]
         else:
-            self.params = dict((k, v) for (k, v) in env.iteritems()
-                               if k != RESOURCE_REGISTRY)
+            self.param_defaults = {}
+
+        if env_fmt.PARAMETERS in env:
+            self.params = env[env_fmt.PARAMETERS]
+        else:
+            self.params = dict((k, v) for (k, v) in six.iteritems(env)
+                               if k not in (env_fmt.PARAMETER_DEFAULTS,
+                                            env_fmt.RESOURCE_REGISTRY))
         self.constraints = {}
+        self.stack_lifecycle_plugins = []
 
     def load(self, env_snippet):
-        self.registry.load(env_snippet.get(RESOURCE_REGISTRY, {}))
-        self.params.update(env_snippet.get('parameters', {}))
+        self.registry.load(env_snippet.get(env_fmt.RESOURCE_REGISTRY, {}))
+        self.params.update(env_snippet.get(env_fmt.PARAMETERS, {}))
+        self.param_defaults.update(
+            env_snippet.get(env_fmt.PARAMETER_DEFAULTS, {}))
+
+    def patch_previous_parameters(self, previous_env, clear_parameters=[]):
+        """This instance of Environment is the new environment where
+        we are reusing as default the previous parameter values.
+        """
+        previous_parameters = copy.deepcopy(previous_env.params)
+        # clear the parameters from the previous set as requested
+        for p in clear_parameters:
+            previous_parameters.pop(p, None)
+
+        # patch the new set of parameters
+        previous_parameters.update(self.params)
+        self.params = previous_parameters
 
     def user_env_as_dict(self):
         """Get the environment as a dict, ready for storing in the db."""
-        return {RESOURCE_REGISTRY: self.registry.as_dict(),
-                PARAMETERS: self.params}
+        return {env_fmt.RESOURCE_REGISTRY: self.registry.as_dict(),
+                env_fmt.PARAMETERS: self.params,
+                env_fmt.PARAMETER_DEFAULTS: self.param_defaults}
 
     def register_class(self, resource_type, resource_class):
         self.registry.register_class(resource_type, resource_class)
@@ -380,6 +504,11 @@ class Environment(object):
     def register_constraint(self, constraint_name, constraint):
         self.constraints[constraint_name] = constraint
 
+    def register_stack_lifecycle_plugin(self, stack_lifecycle_name,
+                                        stack_lifecycle_class):
+        self.stack_lifecycle_plugins.append((stack_lifecycle_name,
+                                             stack_lifecycle_class))
+
     def get_class(self, resource_type, resource_name=None):
         return self.registry.get_class(resource_type, resource_name)
 
@@ -394,6 +523,53 @@ class Environment(object):
     def get_constraint(self, name):
         return self.constraints.get(name)
 
+    def get_stack_lifecycle_plugins(self):
+        return self.stack_lifecycle_plugins
+
+
+def get_child_environment(parent_env, child_params, item_to_remove=None,
+                          child_resource_name=None):
+    """Build a child environment using the parent environment and params.
+
+    This is built from the child_params and the parent env so some
+    resources can use user-provided parameters as if they come from an
+    environment.
+
+    1. resource_registry must be merged (child env should be loaded after the
+       parent env to take presdence).
+    2. child parameters must overwrite the parent's as they won't be relevant
+       in the child template.
+
+    If `child_resource_name` is provided, resources in the registry will be
+    replaced with the contents of the matching child resource plus anything
+    that passes a wildcard match.
+    """
+    def is_flat_params(env_or_param):
+        if env_or_param is None:
+            return False
+        for sect in env_fmt.SECTIONS:
+            if sect in env_or_param:
+                return False
+        return True
+
+    child_env = parent_env.user_env_as_dict()
+    child_env[env_fmt.PARAMETERS] = {}
+    flat_params = is_flat_params(child_params)
+    new_env = Environment()
+    if flat_params and child_params is not None:
+        child_env[env_fmt.PARAMETERS] = child_params
+
+    new_env.load(child_env)
+    if not flat_params and child_params is not None:
+        new_env.load(child_params)
+
+    if item_to_remove is not None:
+        new_env.registry.remove_item(item_to_remove)
+
+    if child_resource_name:
+        new_env.registry.remove_resources_except(child_resource_name)
+    return new_env
+
 
 def read_global_environment(env, env_dir=None):
     if env_dir is None:
@@ -403,22 +579,22 @@ def read_global_environment(env, env_dir=None):
     try:
         env_files = glob.glob(os.path.join(env_dir, '*'))
     except OSError as osex:
-        LOG.error(_('Failed to read %s') % env_dir)
+        LOG.error(_LE('Failed to read %s'), env_dir)
         LOG.exception(osex)
         return
 
     for file_path in env_files:
         try:
             with open(file_path) as env_fd:
-                LOG.info(_('Loading %s') % file_path)
-                env_body = environment_format.parse(env_fd.read())
-                environment_format.default_for_missing(env_body)
+                LOG.info(_LI('Loading %s'), file_path)
+                env_body = env_fmt.parse(env_fd.read())
+                env_fmt.default_for_missing(env_body)
                 env.load(env_body)
         except ValueError as vex:
-            LOG.error(_('Failed to parse %(file_path)s') % {
+            LOG.error(_LE('Failed to parse %(file_path)s'), {
                       'file_path': file_path})
             LOG.exception(vex)
         except IOError as ioex:
-            LOG.error(_('Failed to read %(file_path)s') % {
+            LOG.error(_LE('Failed to read %(file_path)s'), {
                       'file_path': file_path})
             LOG.exception(ioex)
