@@ -18,6 +18,7 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LI
 from heat.common import template_format
 from heat.engine import attributes
 from heat.engine import constraints
@@ -34,21 +35,9 @@ lb_template_default = r'''
     "KeyName" : {
       "Type" : "String"
     },
-    "LbImageId" : {
-      "Type" : "String",
-      "Default" : "Fedora-Cloud-Base-20141203-21.x86_64"
-    },
     "LbFlavor" : {
       "Type" : "String",
       "Default" : "m1.small"
-    },
-    "LBTimeout" : {
-      "Type" : "String",
-      "Default" : "600"
-    },
-    "SecurityGroups" : {
-      "Type" : "CommaDelimitedList",
-      "Default" : []
     }
   },
   "Resources": {
@@ -172,10 +161,9 @@ lb_template_default = r'''
         }
       },
       "Properties": {
-        "ImageId": { "Ref": "LbImageId" },
+        "ImageId": "Fedora-Cloud-Base-20141203-21.x86_64",
         "InstanceType": { "Ref": "LbFlavor" },
         "KeyName": { "Ref": "KeyName" },
-        "SecurityGroups": { "Ref": "SecurityGroups" },
         "UserData": { "Fn::Base64": { "Fn::Join": ["", [
           "#!/bin/bash -v\n",
           "# Helper function\n",
@@ -189,8 +177,7 @@ lb_template_default = r'''
           "/opt/aws/bin/cfn-init -s ",
           { "Ref": "AWS::StackId" },
           "    -r LB_instance ",
-          "    --region ", { "Ref": "AWS::Region" },
-          " || error_exit 'Failed to run cfn-init'\n",
+          "    --region ", { "Ref": "AWS::Region" }, "\n",
 
           "# HAProxy+SELinux, https://www.mankier.com/8/haproxy_selinux \n",
 
@@ -232,7 +219,7 @@ lb_template_default = r'''
       "DependsOn" : "LB_instance",
       "Properties" : {
         "Handle" : {"Ref" : "WaitHandle"},
-        "Timeout" : {"Ref" : "LBTimeout"}
+        "Timeout" : "600"
       }
     }
   },
@@ -250,8 +237,8 @@ lb_template_default = r'''
 # Allow user to provide alternative nested stack template to the above
 loadbalancer_opts = [
     cfg.StrOpt('loadbalancer_template',
-               help=_('Custom template for the built-in '
-                      'loadbalancer nested stack.'))]
+               help='Custom template for the built-in '
+                    'loadbalancer nested stack.')]
 cfg.CONF.register_opts(loadbalancer_opts)
 
 
@@ -273,6 +260,7 @@ class LoadBalancer(stack_resource.StackResource):
     (https://getfedora.org/cloud/download/)
     and apart from installing packages goes through some hoops
     around SELinux due to pecularities of heat-cfntools.
+
     """
 
     PROPERTIES = (
@@ -322,14 +310,14 @@ class LoadBalancer(stack_resource.StackResource):
             _('An application health check for the instances.'),
             schema={
                 HEALTH_CHECK_HEALTHY_THRESHOLD: properties.Schema(
-                    properties.Schema.INTEGER,
+                    properties.Schema.NUMBER,
                     _('The number of consecutive health probe successes '
                       'required before moving the instance to the '
                       'healthy state.'),
                     required=True
                 ),
                 HEALTH_CHECK_INTERVAL: properties.Schema(
-                    properties.Schema.INTEGER,
+                    properties.Schema.NUMBER,
                     _('The approximate interval, in seconds, between '
                       'health checks of an individual instance.'),
                     required=True
@@ -340,12 +328,12 @@ class LoadBalancer(stack_resource.StackResource):
                     required=True
                 ),
                 HEALTH_CHECK_TIMEOUT: properties.Schema(
-                    properties.Schema.INTEGER,
+                    properties.Schema.NUMBER,
                     _('Health probe timeout, in seconds.'),
                     required=True
                 ),
                 HEALTH_CHECK_UNHEALTHY_THRESHOLD: properties.Schema(
-                    properties.Schema.INTEGER,
+                    properties.Schema.NUMBER,
                     _('The number of consecutive health probe failures '
                       'required before moving the instance to the '
                       'unhealthy state'),
@@ -365,13 +353,13 @@ class LoadBalancer(stack_resource.StackResource):
                 properties.Schema.MAP,
                 schema={
                     LISTENER_INSTANCE_PORT: properties.Schema(
-                        properties.Schema.INTEGER,
+                        properties.Schema.NUMBER,
                         _('TCP port on which the instance server is '
                           'listening.'),
                         required=True
                     ),
                     LISTENER_LOAD_BALANCER_PORT: properties.Schema(
-                        properties.Schema.INTEGER,
+                        properties.Schema.NUMBER,
                         _('The external load balancer port number.'),
                         required=True
                     ),
@@ -408,9 +396,9 @@ class LoadBalancer(stack_resource.StackResource):
             implemented=False
         ),
         SECURITY_GROUPS: properties.Schema(
-            properties.Schema.LIST,
-            _('List of Security Groups assigned on current LB.'),
-            update_allowed=True
+            properties.Schema.STRING,
+            _('Not Implemented.'),
+            implemented=False
         ),
         SUBNETS: properties.Schema(
             properties.Schema.LIST,
@@ -422,26 +410,21 @@ class LoadBalancer(stack_resource.StackResource):
     attributes_schema = {
         CANONICAL_HOSTED_ZONE_NAME: attributes.Schema(
             _("The name of the hosted zone that is associated with the "
-              "LoadBalancer."),
-            type=attributes.Schema.STRING
+              "LoadBalancer.")
         ),
         CANONICAL_HOSTED_ZONE_NAME_ID: attributes.Schema(
             _("The ID of the hosted zone name that is associated with the "
-              "LoadBalancer."),
-            type=attributes.Schema.STRING
+              "LoadBalancer.")
         ),
         DNS_NAME: attributes.Schema(
-            _("The DNS name for the LoadBalancer."),
-            type=attributes.Schema.STRING
+            _("The DNS name for the LoadBalancer.")
         ),
         SOURCE_SECURITY_GROUP_GROUP_NAME: attributes.Schema(
             _("The security group that you can use as part of your inbound "
-              "rules for your LoadBalancer's back-end instances."),
-            type=attributes.Schema.STRING
+              "rules for your LoadBalancer's back-end instances.")
         ),
         SOURCE_SECURITY_GROUP_OWNER_ALIAS: attributes.Schema(
-            _("Owner of the source security group."),
-            type=attributes.Schema.STRING
+            _("Owner of the source security group.")
         ),
     }
 
@@ -504,7 +487,7 @@ backend servers
         nova_cp = self.client_plugin('nova')
         for i in instances or []:
             ip = nova_cp.server_to_ipaddress(i) or '0.0.0.0'
-            LOG.debug('haproxy server:%s', ip)
+            LOG.debug('haproxy server:%s' % ip)
             servers.append('%sserver server%d %s:%s%s' % (spaces, n,
                                                           ip, inst_port,
                                                           check))
@@ -525,7 +508,7 @@ backend servers
     def get_parsed_template(self):
         if cfg.CONF.loadbalancer_template:
             with open(cfg.CONF.loadbalancer_template) as templ_fd:
-                LOG.info('Using custom loadbalancer template %s',
+                LOG.info(_LI('Using custom loadbalancer template %s'),
                          cfg.CONF.loadbalancer_template)
                 contents = templ_fd.read()
         else:
@@ -535,12 +518,13 @@ backend servers
     def child_params(self):
         params = {}
 
-        params['SecurityGroups'] = self.properties[self.SECURITY_GROUPS]
         # If the owning stack defines KeyName, we use that key for the nested
         # template, otherwise use no key
-        for magic_param in ('KeyName', 'LbFlavor', 'LBTimeout', 'LbImageId'):
-            if magic_param in self.stack.parameters:
-                params[magic_param] = self.stack.parameters[magic_param]
+        if 'KeyName' in self.stack.parameters:
+            params['KeyName'] = self.stack.parameters['KeyName']
+
+        if 'LbFlavor' in self.stack.parameters:
+            params['LbFlavor'] = self.stack.parameters['LbFlavor']
 
         return params
 
@@ -568,12 +552,11 @@ backend servers
         return self.create_with_template(templ, params)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        """Re-generate the Metadata.
-
-        Save it to the db.
-        Rely on the cfn-hup to reconfigure HAProxy.
-        """
-
+        '''
+        re-generate the Metadata
+        save it to the db.
+        rely on the cfn-hup to reconfigure HAProxy
+        '''
         new_props = json_snippet.properties(self.properties_schema,
                                             self.context)
 
@@ -593,18 +576,17 @@ backend servers
 
             self.nested()['LB_instance'].metadata_set(md)
 
-        if self.SECURITY_GROUPS in prop_diff:
-            templ = self.child_template()
-            params = self.child_params()
-            params['SecurityGroups'] = new_props[self.SECURITY_GROUPS]
-            self.update_with_template(templ, params)
-
     def check_update_complete(self, updater):
         """Because we are not calling update_with_template, return True."""
         return True
 
+    def handle_delete(self):
+        return self.delete_nested()
+
     def validate(self):
-        """Validate any of the provided params."""
+        '''
+        Validate any of the provided params
+        '''
         res = super(LoadBalancer, self).validate()
         if res:
             return res
@@ -622,17 +604,15 @@ backend servers
                 return {'Error':
                         'Interval must be larger than Timeout'}
 
-    def get_reference_id(self):
+    def FnGetRefId(self):
         return six.text_type(self.name)
 
     def _resolve_attribute(self, name):
-        """We don't really support any of these yet."""
+        '''
+        We don't really support any of these yet.
+        '''
         if name == self.DNS_NAME:
-            try:
-                return self.get_output('PublicIp')
-            except exception.NotFound:
-                raise exception.InvalidTemplateAttribute(resource=self.name,
-                                                         key=name)
+            return self.get_output('PublicIp')
         elif name in self.attributes_schema:
             # Not sure if we should return anything for the other attribs
             # since they aren't really supported in any meaningful way
