@@ -11,23 +11,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
 import json
 
 import mock
-from oslo_utils import timeutils
+from oslo_config import cfg
 import six
 
 from heat.common import exception
 from heat.common import grouputils
 from heat.common import template_format
 from heat.engine.clients.os import nova
-from heat.engine import resource
+from heat.engine import function
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.tests.autoscaling import inline_templates
 from heat.tests import common
-from heat.tests.openstack.nova import fakes as fakes_nova
+from heat.tests.nova import fakes as fakes_nova
 from heat.tests import utils
 
 
@@ -35,6 +34,11 @@ as_template = inline_templates.as_template
 
 
 class TestAutoScalingGroupValidation(common.HeatTestCase):
+    def setUp(self):
+        super(TestAutoScalingGroupValidation, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
     def validate_scaling_group(self, t, stack, resource_name):
         # create the launch configuration resource
         conf = stack['LaunchConfig']
@@ -179,8 +183,8 @@ class TestAutoScalingGroupValidation(common.HeatTestCase):
         mock_server.security_groups = [{u'name': u'hth_test'}]
         if not_found:
             self.patchobject(nova.NovaClientPlugin, 'get_server',
-                             side_effect=exception.EntityNotFound(
-                                 entity='Server', name='5678'))
+                             side_effect=exception.ServerNotFound(
+                                 server='5678'))
         else:
             self.patchobject(nova.NovaClientPlugin, 'get_server',
                              return_value=mock_server)
@@ -216,10 +220,8 @@ class TestAutoScalingGroupValidation(common.HeatTestCase):
         rsrc = stack['WebServerGroup']
         self._stub_nova_server_get(not_found=True)
         self.m.ReplayAll()
-        msg = ("Property error: "
-               "Resources.WebServerGroup.Properties.InstanceId: "
-               "Error validating value '5678': The Server (5678) could "
-               "not be found.")
+        msg = ("Property error : WebServerGroup: InstanceId Error validating "
+               "value '5678': The server (5678) could not be found")
         exc = self.assertRaises(exception.StackValidationFailed,
                                 rsrc.validate)
         self.assertIn(msg, six.text_type(exc))
@@ -231,8 +233,8 @@ class TestScalingGroupTags(common.HeatTestCase):
     def setUp(self):
         super(TestScalingGroupTags, self).setUp()
         t = template_format.parse(as_template)
-        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = self.stack['WebServerGroup']
+        stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = stack['WebServerGroup']
 
     def test_tags_default(self):
         expected = [{'Key': 'metering.groupname',
@@ -270,6 +272,11 @@ class TestInitialGroupSize(common.HeatTestCase):
         ('14n', dict(mins=1, maxs=4, desired=None, expected=1)),
     ]
 
+    def setUp(self):
+        super(TestInitialGroupSize, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
     def test_initial_size(self):
         t = template_format.parse(as_template)
         properties = t['Resources']['WebServerGroup']['Properties']
@@ -286,121 +293,63 @@ class TestInitialGroupSize(common.HeatTestCase):
 class TestGroupAdjust(common.HeatTestCase):
     def setUp(self):
         super(TestGroupAdjust, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
 
         t = template_format.parse(as_template)
-        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = self.stack['WebServerGroup']
+        stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = stack['WebServerGroup']
         self.stub_ImageConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_SnapshotConstraint_validate()
         self.assertIsNone(self.group.validate())
 
     def test_scaling_policy_cooldown_toosoon(self):
-        dont_call = self.patchobject(self.group, 'resize')
-        self.patchobject(self.group, '_check_scaling_allowed',
-                         side_effect=resource.NoActionRequired)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group.adjust, 1)
+        """If _cooldown_inprogress() returns True don't progress."""
+
+        dont_call = self.patchobject(grouputils, 'get_members')
+        with mock.patch.object(self.group, '_cooldown_inprogress',
+                               return_value=True):
+            self.group.adjust(1)
         self.assertEqual([], dont_call.call_args_list)
 
     def test_scaling_same_capacity(self):
-        """Don't resize when capacity is the same."""
+        """Alway resize even if the capacity is the same."""
         self.patchobject(grouputils, 'get_size', return_value=3)
         resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
+        cd_stamp = self.patchobject(self.group, '_cooldown_timestamp')
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.assertRaises(resource.NoActionRequired,
-                          self.group.adjust, 3,
-                          adjustment_type='ExactCapacity')
-        expected_notifies = []
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        self.assertEqual(0, resize.call_count)
-        self.assertEqual(0, finished_scaling.call_count)
-
-    def test_scaling_update_in_progress(self):
-        """Don't resize when update in progress"""
-        self.group.state_set('UPDATE', 'IN_PROGRESS')
-        resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.assertRaises(resource.NoActionRequired,
-                          self.group.adjust, 3,
-                          adjustment_type='ExactCapacity')
-        expected_notifies = []
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        self.assertEqual(0, resize.call_count)
-        self.assertEqual(0, finished_scaling.call_count)
-
-    def test_scale_up_min_adjustment(self):
-        self.patchobject(grouputils, 'get_size', return_value=1)
-        resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.group.adjust(33, adjustment_type='PercentChangeInCapacity',
-                          min_adjustment_step=2)
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
+        self.group.adjust(3, adjustment_type='ExactCapacity')
 
         expected_notifies = [
             mock.call(
-                capacity=1, suffix='start',
-                adjustment_type='PercentChangeInCapacity',
+                capacity=3, suffix='start',
+                adjustment_type='ExactCapacity',
                 groupname=u'WebServerGroup',
                 message=u'Start resizing the group WebServerGroup',
-                adjustment=33,
+                adjustment=3,
                 stack=self.group.stack),
             mock.call(
                 capacity=3, suffix='end',
-                adjustment_type='PercentChangeInCapacity',
+                adjustment_type='ExactCapacity',
                 groupname=u'WebServerGroup',
                 message=u'End resizing the group WebServerGroup',
-                adjustment=33,
+                adjustment=3,
                 stack=self.group.stack)]
 
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(3)
-        finished_scaling.assert_called_once_with(
-            None,
-            'PercentChangeInCapacity : 33',
-            size_changed=True)
-
-    def test_scale_down_min_adjustment(self):
-        self.patchobject(grouputils, 'get_size', return_value=5)
-        resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.group.adjust(-33, adjustment_type='PercentChangeInCapacity',
-                          min_adjustment_step=2)
-
-        expected_notifies = [
-            mock.call(
-                capacity=5, suffix='start',
-                adjustment_type='PercentChangeInCapacity',
-                groupname=u'WebServerGroup',
-                message=u'Start resizing the group WebServerGroup',
-                adjustment=-33,
-                stack=self.group.stack),
-            mock.call(
-                capacity=3, suffix='end',
-                adjustment_type='PercentChangeInCapacity',
-                groupname=u'WebServerGroup',
-                message=u'End resizing the group WebServerGroup',
-                adjustment=-33,
-                stack=self.group.stack)]
-
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        resize.assert_called_once_with(3)
-        finished_scaling.assert_called_once_with(
-            None,
-            'PercentChangeInCapacity : -33',
-            size_changed=True)
+        cd_stamp.assert_called_once_with('ExactCapacity : 3')
 
     def test_scaling_policy_cooldown_ok(self):
-        self.patchobject(grouputils, 'get_size', return_value=0)
+        self.patchobject(grouputils, 'get_members', return_value=[])
         resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
+        cd_stamp = self.patchobject(self.group, '_cooldown_timestamp')
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
         self.group.adjust(1)
 
         expected_notifies = [
@@ -420,18 +369,15 @@ class TestGroupAdjust(common.HeatTestCase):
 
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(1)
-        finished_scaling.assert_called_once_with(None,
-                                                 'ChangeInCapacity : 1',
-                                                 size_changed=True)
-        grouputils.get_size.assert_called_once_with(self.group)
+        cd_stamp.assert_called_once_with('ChangeInCapacity : 1')
 
     def test_scaling_policy_resize_fail(self):
-        self.patchobject(grouputils, 'get_size', return_value=0)
+        self.patchobject(grouputils, 'get_members', return_value=[])
         self.patchobject(self.group, 'resize',
                          side_effect=ValueError('test error'))
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.patchobject(self.group, '_finished_scaling')
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
         self.assertRaises(ValueError, self.group.adjust, 1)
 
         expected_notifies = [
@@ -451,48 +397,17 @@ class TestGroupAdjust(common.HeatTestCase):
                 stack=self.group.stack)]
 
         self.assertEqual(expected_notifies, notify.call_args_list)
-        grouputils.get_size.assert_called_with(self.group)
-
-    def test_notification_send_if_resize_failed(self):
-        """If resize failed, the capacity of group might have been changed"""
-        self.patchobject(grouputils, 'get_size', side_effect=[3, 4])
-        self.patchobject(self.group, 'resize',
-                         side_effect=ValueError('test error'))
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.patchobject(self.group, '_finished_scaling')
-
-        self.assertRaises(ValueError, self.group.adjust,
-                          5, adjustment_type='ExactCapacity')
-
-        expected_notifies = [
-            mock.call(
-                capacity=3, suffix='start',
-                adjustment_type='ExactCapacity',
-                groupname=u'WebServerGroup',
-                message=u'Start resizing the group WebServerGroup',
-                adjustment=5,
-                stack=self.group.stack),
-            mock.call(
-                capacity=4, suffix='error',
-                adjustment_type='ExactCapacity',
-                groupname=u'WebServerGroup',
-                message=u'test error',
-                adjustment=5,
-                stack=self.group.stack)]
-
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        self.group.resize.assert_called_once_with(5)
-        grouputils.get_size.assert_has_calls([mock.call(self.group),
-                                              mock.call(self.group)])
 
 
 class TestGroupCrud(common.HeatTestCase):
     def setUp(self):
         super(TestGroupCrud, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
         t = template_format.parse(as_template)
-        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = self.stack['WebServerGroup']
+        stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = stack['WebServerGroup']
         self.assertIsNone(self.group.validate())
 
     def test_handle_create(self):
@@ -506,35 +421,36 @@ class TestGroupCrud(common.HeatTestCase):
 
     def test_handle_update_desired_cap(self):
         self.group._try_rolling_update = mock.Mock(return_value=None)
-        self.group.resize = mock.Mock(return_value=None)
-        props = {'DesiredCapacity': 4,
-                 'MinSize': 0,
-                 'MaxSize': 6}
-        self.group._get_new_capacity = mock.Mock(return_value=4)
+        self.group.adjust = mock.Mock(return_value=None)
+
+        props = {'DesiredCapacity': 4}
         defn = rsrc_defn.ResourceDefinition(
             'nopayload',
             'AWS::AutoScaling::AutoScalingGroup',
             props)
 
         self.group.handle_update(defn, None, props)
-        self.group.resize.assert_called_once_with(4)
+
+        self.group.adjust.assert_called_once_with(
+            4, adjustment_type='ExactCapacity')
         self.group._try_rolling_update.assert_called_once_with(props)
 
     def test_handle_update_desired_nocap(self):
         self.group._try_rolling_update = mock.Mock(return_value=None)
-        self.group.resize = mock.Mock(return_value=None)
+        self.group.adjust = mock.Mock(return_value=None)
         get_size = self.patchobject(grouputils, 'get_size')
-        get_size.return_value = 4
+        get_size.return_value = 6
 
-        props = {'MinSize': 0,
-                 'MaxSize': 6}
+        props = {'Tags': []}
         defn = rsrc_defn.ResourceDefinition(
             'nopayload',
             'AWS::AutoScaling::AutoScalingGroup',
             props)
 
         self.group.handle_update(defn, None, props)
-        self.group.resize.assert_called_once_with(4)
+
+        self.group.adjust.assert_called_once_with(
+            6, adjustment_type='ExactCapacity')
         self.group._try_rolling_update.assert_called_once_with(props)
 
     def test_conf_properties_vpc_zone(self):
@@ -561,7 +477,7 @@ class TestGroupCrud(common.HeatTestCase):
     def test_update_in_failed(self):
         self.group.state_set('CREATE', 'FAILED')
         # to update the failed asg
-        self.group.resize = mock.Mock(return_value=None)
+        self.group.adjust = mock.Mock(return_value=None)
 
         new_defn = rsrc_defn.ResourceDefinition(
             'asg', 'AWS::AutoScaling::AutoScalingGroup',
@@ -572,7 +488,8 @@ class TestGroupCrud(common.HeatTestCase):
              'DesiredCapacity': 2})
 
         self.group.handle_update(new_defn, None, None)
-        self.group.resize.assert_called_once_with(2)
+        self.group.adjust.assert_called_once_with(
+            2, adjustment_type='ExactCapacity')
 
 
 def asg_tmpl_with_bad_updt_policy():
@@ -608,6 +525,8 @@ class RollingUpdatePolicyTest(common.HeatTestCase):
         self.stub_ImageConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_SnapshotConstraint_validate()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://127.0.0.1:8000/v1/waitcondition')
 
     def test_parse_without_update_policy(self):
         tmpl = template_format.parse(inline_templates.as_template)
@@ -630,8 +549,7 @@ class RollingUpdatePolicyTest(common.HeatTestCase):
         self.assertEqual(1, len(grp.update_policy))
         self.assertIn('AutoScalingRollingUpdate', grp.update_policy)
         policy = grp.update_policy['AutoScalingRollingUpdate']
-        self.assertIsNotNone(policy)
-        self.assertGreater(len(policy), 0)
+        self.assertTrue(policy and len(policy) > 0)
         self.assertEqual(1, int(policy['MinInstancesInService']))
         self.assertEqual(tmpl_batch_sz, int(policy['MaxBatchSize']))
         self.assertEqual('PT1S', policy['PauseTime'])
@@ -646,8 +564,7 @@ class RollingUpdatePolicyTest(common.HeatTestCase):
         self.assertEqual(1, len(grp.update_policy))
         self.assertIn('AutoScalingRollingUpdate', grp.update_policy)
         policy = grp.update_policy['AutoScalingRollingUpdate']
-        self.assertIsNotNone(policy)
-        self.assertGreater(len(policy), 0)
+        self.assertTrue(policy and len(policy) > 0)
         self.assertEqual(0, int(policy['MinInstancesInService']))
         self.assertEqual(1, int(policy['MaxBatchSize']))
         self.assertEqual('PT0S', policy['PauseTime'])
@@ -675,6 +592,8 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
         super(RollingUpdatePolicyDiffTest, self).setUp()
         self.fc = fakes_nova.FakeClient()
         self.stub_keystoneclient(username='test_stack.CfnLBUser')
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://127.0.0.1:8000/v1/waitcondition')
 
     def validate_update_policy_diff(self, current, updated):
         # load current stack
@@ -684,7 +603,7 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
 
         # get the json snippet for the current InstanceGroup resource
         current_grp = current_stack['WebServerGroup']
-        current_snippets = dict((n, r.frozen_definition())
+        current_snippets = dict((n, r.parsed_template())
                                 for n, r in current_stack.items())
         current_grp_json = current_snippets[current_grp.name]
 
@@ -696,19 +615,29 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
         # get the updated json snippet for the InstanceGroup resource in the
         # context of the current stack
         updated_grp = updated_stack['WebServerGroup']
-        updated_grp_json = updated_grp.t.freeze()
+        updated_grp_json = function.resolve(updated_grp.t)
 
         # identify the template difference
         tmpl_diff = updated_grp.update_template_diff(
             updated_grp_json, current_grp_json)
-        self.assertTrue(tmpl_diff.update_policy_changed())
+        updated_policy = (updated_grp.t['UpdatePolicy']
+                          if 'UpdatePolicy' in updated_grp.t else None)
+        expected = {u'UpdatePolicy': updated_policy}
+        self.assertEqual(expected, tmpl_diff)
 
         # test application of the new update policy in handle_update
+        update_snippet = rsrc_defn.ResourceDefinition(
+            current_grp.name,
+            current_grp.type(),
+            properties=updated_grp.t['Properties'],
+            update_policy=updated_policy)
         current_grp._try_rolling_update = mock.MagicMock()
-        current_grp.resize = mock.MagicMock()
-        current_grp.handle_update(updated_grp_json, tmpl_diff, None)
-        self.assertEqual(updated_grp_json._update_policy or {},
-                         current_grp.update_policy.data)
+        current_grp.adjust = mock.MagicMock()
+        current_grp.handle_update(update_snippet, tmpl_diff, None)
+        if updated_policy is None:
+            self.assertEqual({}, current_grp.update_policy.data)
+        else:
+            self.assertEqual(updated_policy, current_grp.update_policy.data)
 
     def test_update_policy_added(self):
         self.validate_update_policy_diff(inline_templates.as_template,
@@ -727,121 +656,3 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
     def test_update_policy_removed(self):
         self.validate_update_policy_diff(asg_tmpl_with_updt_policy(),
                                          inline_templates.as_template)
-
-
-class TestCooldownMixin(common.HeatTestCase):
-    def setUp(self):
-        super(TestCooldownMixin, self).setUp()
-        t = template_format.parse(inline_templates.as_template)
-        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.stack.store()
-        self.group = self.stack['WebServerGroup']
-        self.group.state_set('CREATE', 'COMPLETE')
-
-    def test_cooldown_is_in_progress_toosoon(self):
-        cooldown_end = timeutils.utcnow() + datetime.timedelta(seconds=60)
-        previous_meta = {'cooldown_end': {
-            cooldown_end.isoformat(): 'ChangeInCapacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_cooldown_is_in_progress_toosoon_legacy(self):
-        now = timeutils.utcnow()
-        previous_meta = {'cooldown': {
-            now.isoformat(): 'ChangeInCapacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_cooldown_is_in_progress_scaling_unfinished(self):
-        previous_meta = {'scaling_in_progress': True}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_scaling_not_in_progress_legacy(self):
-        awhile_ago = timeutils.utcnow() - datetime.timedelta(seconds=100)
-        previous_meta = {
-            'cooldown': {
-                awhile_ago.isoformat(): 'ChangeInCapacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-
-    def test_scaling_not_in_progress(self):
-        awhile_after = timeutils.utcnow() + datetime.timedelta(seconds=60)
-        previous_meta = {
-            'cooldown_end': {
-                awhile_after.isoformat(): 'ChangeInCapacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        timeutils.set_time_override()
-        timeutils.advance_time_seconds(100)
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-        timeutils.clear_time_override()
-
-    def test_scaling_policy_cooldown_zero(self):
-        now = timeutils.utcnow()
-        previous_meta = {
-            'cooldown_end': {
-                now.isoformat(): 'ChangeInCapacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-
-    def test_scaling_policy_cooldown_none(self):
-        now = timeutils.utcnow()
-        previous_meta = {
-            'cooldown_end': {
-                now.isoformat(): 'ChangeInCapacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(None))
-
-    def test_metadata_is_written(self):
-        nowish = timeutils.utcnow()
-        reason = 'cool as'
-        meta_set = self.patchobject(self.group, 'metadata_set')
-        self.patchobject(timeutils, 'utcnow', return_value=nowish)
-        self.group._finished_scaling(60, reason)
-        cooldown_end = nowish + datetime.timedelta(seconds=60)
-        meta_set.assert_called_once_with(
-            {'cooldown_end': {cooldown_end.isoformat(): reason},
-             'scaling_in_progress': False})
-
-    def test_metadata_is_written_update(self):
-        nowish = timeutils.utcnow()
-        reason = 'cool as'
-        prev_cooldown_end = nowish + datetime.timedelta(seconds=100)
-        previous_meta = {
-            'cooldown_end': {
-                prev_cooldown_end.isoformat(): 'ChangeInCapacity : 1'
-            }
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        meta_set = self.patchobject(self.group, 'metadata_set')
-        self.patchobject(timeutils, 'utcnow', return_value=nowish)
-        self.group._finished_scaling(60, reason)
-        meta_set.assert_called_once_with(
-            {'cooldown_end': {prev_cooldown_end.isoformat(): reason},
-             'scaling_in_progress': False})
