@@ -12,6 +12,7 @@
 #    under the License.
 
 import copy
+
 import six
 
 from heat.common import exception
@@ -28,11 +29,13 @@ from heat.engine import resource
 from heat.engine import resources
 from heat.engine import rsrc_defn
 from heat.engine import template
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import generic_resource as generic_rsrc
-from heat.tests import test_parser
 from heat.tests import utils
 
+empty_template = template_format.parse('''{
+  "HeatTemplateFormatVersion" : "2012-12-12",
+}''')
 
 hot_tpl_empty = template_format.parse('''
 heat_template_version: 2013-05-23
@@ -42,6 +45,10 @@ hot_juno_tpl_empty = template_format.parse('''
 heat_template_version: 2014-10-16
 ''')
 
+hot_kilo_tpl_empty = template_format.parse('''
+heat_template_version: 2015-04-30
+''')
+
 hot_tpl_empty_sections = template_format.parse('''
 heat_template_version: 2013-05-23
 parameters:
@@ -87,7 +94,7 @@ class DummyClass(object):
         self.metadata = metadata
 
 
-class HOTemplateTest(HeatTestCase):
+class HOTemplateTest(common.HeatTestCase):
     """Test processing of HOT templates."""
 
     @staticmethod
@@ -156,6 +163,21 @@ class HOTemplateTest(HeatTestCase):
         tmpl = parser.Template(hot_tpl)
         self.assertEqual(expected, tmpl[tmpl.RESOURCES])
 
+    def test_translate_resources_bad_no_data(self):
+        """Test translation of resources without any mapping."""
+
+        hot_tpl = template_format.parse("""
+        heat_template_version: 2013-05-23
+        resources:
+          resource1:
+        """)
+
+        tmpl = parser.Template(hot_tpl)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.__getitem__, tmpl.RESOURCES)
+        self.assertEqual('Each resource must contain a type key.',
+                         six.text_type(error))
+
     def test_translate_resources_bad_type(self):
         """Test translation of resources including invalid keyword."""
 
@@ -175,7 +197,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Type" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
@@ -199,11 +222,31 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Properties" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
 
+    def test_translate_resources_resources_without_name(self):
+        hot_tpl = template_format.parse('''
+        heat_template_version: 2013-05-23
+        resources:
+          type: AWS::EC2::Instance
+          properties:
+            property1: value1
+          metadata:
+            foo: bar
+          depends_on: dummy
+          deletion_policy: dummy
+        ''')
+        tmpl = parser.Template(hot_tpl)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.__getitem__, tmpl.RESOURCES)
+        self.assertEqual('"resources" must contain a map of resource maps. '
+                         'Found a [%s] instead' % six.text_type,
+                         six.text_type(error))
+
     def test_translate_resources_bad_metadata(self):
         """Test translation of resources including invalid keyword."""
 
@@ -223,7 +266,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Metadata" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
@@ -247,12 +291,13 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"DependsOn" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
 
-    def test_translate_resources_bad_deletion_polciy(self):
+    def test_translate_resources_bad_deletion_policy(self):
         """Test translation of resources including invalid keyword."""
 
         hot_tpl = template_format.parse('''
@@ -271,7 +316,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"DeletionPolicy" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
@@ -295,7 +341,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"UpdatePolicy" is not a valid keyword '
                          'inside a resource definition\'',
                          six.text_type(err))
@@ -316,6 +363,38 @@ class HOTemplateTest(HeatTestCase):
         tmpl = parser.Template(hot_tpl)
         self.assertEqual(expected, tmpl[tmpl.OUTPUTS])
 
+    def test_translate_outputs_bad_no_data(self):
+        """Test translation of outputs without any mapping."""
+
+        hot_tpl = template_format.parse("""
+        heat_template_version: 2013-05-23
+        outputs:
+          output1:
+        """)
+
+        tmpl = parser.Template(hot_tpl)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.__getitem__, tmpl.OUTPUTS)
+        self.assertEqual('Each output must contain a value key.',
+                         six.text_type(error))
+
+    def test_translate_outputs_bad_without_name(self):
+        """Test translation of outputs without name."""
+
+        hot_tpl = template_format.parse("""
+        heat_template_version: 2013-05-23
+        outputs:
+          description: wrong output
+          value: value1
+        """)
+
+        tmpl = parser.Template(hot_tpl)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.__getitem__, tmpl.OUTPUTS)
+        self.assertEqual('"outputs" must contain a map of output maps. '
+                         'Found a [%s] instead' % six.text_type,
+                         six.text_type(error))
+
     def test_translate_outputs_bad_description(self):
         """Test translation of outputs into internal engine format."""
 
@@ -328,7 +407,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.OUTPUTS)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.OUTPUTS)
         self.assertIn('Description', six.text_type(err))
 
     def test_translate_outputs_bad_value(self):
@@ -343,7 +423,8 @@ class HOTemplateTest(HeatTestCase):
         ''')
 
         tmpl = parser.Template(hot_tpl)
-        err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.OUTPUTS)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                tmpl.__getitem__, tmpl.OUTPUTS)
         self.assertIn('Value', six.text_type(err))
 
     def test_resource_group_list_join(self):
@@ -365,7 +446,7 @@ class HOTemplateTest(HeatTestCase):
         tmpl = parser.Template(hot_tpl)
         stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
         snippet = {'list_join': ["\n", {'get_attr': ['rg', 'name']}]}
-        self.assertEqual(self.resolve(snippet, tmpl, stack), '')
+        self.assertEqual('', self.resolve(snippet, tmpl, stack))
 
     def test_str_replace(self):
         """Test str_replace function."""
@@ -510,6 +591,126 @@ class HOTemplateTest(HeatTestCase):
 
         self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl, stack))
 
+    def test_repeat(self):
+        """Test repeat function."""
+        snippet = {'repeat': {'template': 'this is %var%',
+                              'for_each': {'%var%': ['a', 'b', 'c']}}}
+        snippet_resolved = ['this is a', 'this is b', 'this is c']
+
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
+
+    def test_repeat_dict_template(self):
+        """Test repeat function with a dictionary as a template."""
+        snippet = {'repeat': {'template': {'key-%var%': 'this is %var%'},
+                              'for_each': {'%var%': ['a', 'b', 'c']}}}
+        snippet_resolved = [{'key-a': 'this is a'},
+                            {'key-b': 'this is b'},
+                            {'key-c': 'this is c'}]
+
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
+
+    def test_repeat_list_template(self):
+        """Test repeat function with a list as a template."""
+        snippet = {'repeat': {'template': ['this is %var%', 'static'],
+                              'for_each': {'%var%': ['a', 'b', 'c']}}}
+        snippet_resolved = [['this is a', 'static'],
+                            ['this is b', 'static'],
+                            ['this is c', 'static']]
+
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
+
+    def test_repeat_multi_list(self):
+        """Test repeat function with multiple input lists."""
+        snippet = {'repeat': {'template': 'this is %var1%-%var2%',
+                              'for_each': {'%var1%': ['a', 'b', 'c'],
+                                           '%var2%': ['1', '2']}}}
+        snippet_resolved = ['this is a-1', 'this is b-1', 'this is c-1',
+                            'this is a-2', 'this is b-2', 'this is c-2']
+
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        result = self.resolve(snippet, tmpl)
+        self.assertEqual(len(result), len(snippet_resolved))
+        for item in result:
+            self.assertIn(item, snippet_resolved)
+
+    def test_repeat_bad_args(self):
+        """
+        Test that the repeat function reports a proper error when missing
+        or invalid arguments.
+        """
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        # missing for_each
+        snippet = {'repeat': {'template': 'this is %var%'}}
+        self.assertRaises(KeyError, self.resolve, snippet, tmpl)
+
+        # misspelled for_each
+        snippet = {'repeat': {'template': 'this is %var%',
+                              'foreach': {'%var%': ['a', 'b', 'c']}}}
+        self.assertRaises(KeyError, self.resolve, snippet, tmpl)
+
+        # for_each is not a map
+        snippet = {'repeat': {'template': 'this is %var%',
+                              'for_each': '%var%'}}
+        self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+
+        # value given to for_each entry is not a list
+        snippet = {'repeat': {'template': 'this is %var%',
+                              'for_each': {'%var%': 'a'}}}
+        self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+
+        # misspelled template
+        snippet = {'repeat': {'templte': 'this is %var%',
+                              'for_each': {'%var%': ['a', 'b', 'c']}}}
+        self.assertRaises(KeyError, self.resolve, snippet, tmpl)
+
+    def test_digest(self):
+        snippet = {'digest': ['md5', 'foobar']}
+        snippet_resolved = '3858f62230ac3c915f300c664312c63f'
+
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+        self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
+
+    def test_digest_invalid_types(self):
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        invalid_snippets = [
+            {'digest': 'invalid'},
+            {'digest': {'foo': 'invalid'}},
+            {'digest': [123]},
+        ]
+        for snippet in invalid_snippets:
+            exc = self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+            self.assertIn('must be a list of strings', six.text_type(exc))
+
+    def test_digest_incorrect_number_arguments(self):
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        invalid_snippets = [
+            {'digest': []},
+            {'digest': ['foo']},
+            {'digest': ['md5']},
+            {'digest': ['md5', 'foo', 'bar']},
+        ]
+        for snippet in invalid_snippets:
+            exc = self.assertRaises(ValueError, self.resolve, snippet, tmpl)
+            self.assertIn('usage: ["<algorithm>", "<value>"]',
+                          six.text_type(exc))
+
+    def test_digest_invalid_algorithm(self):
+        tmpl = parser.Template(hot_kilo_tpl_empty)
+
+        snippet = {'digest': ['invalid_algorithm', 'foobar']}
+        exc = self.assertRaises(ValueError, self.resolve, snippet, tmpl)
+        self.assertIn('Algorithm must be one of', six.text_type(exc))
+
     def test_prevent_parameters_access(self):
         """
         Test that the parameters section can't be accessed using the template
@@ -529,11 +730,11 @@ class HOTemplateTest(HeatTestCase):
 
         err_str = "can not be accessed directly"
 
-        #Hot template test
+        # Hot template test
         keyError = self.assertRaises(KeyError, tmpl.__getitem__, 'parameters')
         self.assertIn(err_str, six.text_type(keyError))
 
-        #CFN template test
+        # CFN template test
         keyError = self.assertRaises(KeyError, tmpl.__getitem__, 'Parameters')
         self.assertIn(err_str, six.text_type(keyError))
 
@@ -594,7 +795,8 @@ class HOTemplateTest(HeatTestCase):
                                              parser.Template(hot_tpl_empty))
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
-                             parent_resource=parent_resource)
+                             parent_resource='parent')
+        stack._parent_resource = parent_resource
         self.assertEqual({"foo": "bar"},
                          self.resolve(metadata_snippet, stack.t, stack))
         self.assertEqual('Retain',
@@ -619,7 +821,8 @@ class HOTemplateTest(HeatTestCase):
 
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
-                             parent_resource=parent_resource)
+                             parent_resource='parent')
+        stack._parent_resource = parent_resource
         self.assertEqual('Retain',
                          self.resolve(deletion_policy_snippet, stack.t, stack))
 
@@ -644,7 +847,8 @@ class HOTemplateTest(HeatTestCase):
                                              parser.Template(hot_tpl_empty))
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
-                             parent_resource=parent_resource)
+                             parent_resource='parent')
+        stack._parent_resource = parent_resource
         self.assertEqual('Delete', self.resolve(snippet, stack.t, stack))
 
     def test_removed_function(self):
@@ -671,8 +875,6 @@ class HOTemplateTest(HeatTestCase):
             deletion_policy: Retain
             update_policy:
               foo: bar
-          resource2:
-            type: AWS::EC2::Instance
         ''')
         source = parser.Template(hot_tpl)
         empty = parser.Template(copy.deepcopy(hot_tpl_empty))
@@ -684,8 +886,22 @@ class HOTemplateTest(HeatTestCase):
         self.assertEqual(hot_tpl['resources'], empty.t['resources'])
 
 
-class StackTest(test_parser.StackTest):
+class HotStackTest(common.HeatTestCase):
     """Test stack function when stack was created from HOT template."""
+    def setUp(self):
+        super(HotStackTest, self).setUp()
+
+        self.tmpl = template.Template(copy.deepcopy(empty_template))
+        self.ctx = utils.dummy_context()
+
+        resource._register_class('GenericResourceType',
+                                 generic_rsrc.GenericResource)
+        resource._register_class('ResourceWithPropsType',
+                                 generic_rsrc.ResourceWithProps)
+        resource._register_class('ResourceWithComplexAttributesType',
+                                 generic_rsrc.ResourceWithComplexAttributes)
+        resource._register_class('ResWithComplexPropsAndAttrs',
+                                 generic_rsrc.ResWithComplexPropsAndAttrs)
 
     def resolve(self, snippet):
         return function.resolve(self.stack.t.parse(self.stack, snippet))
@@ -763,12 +979,12 @@ class StackTest(test_parser.StackTest):
     def test_set_param_id(self):
         tmpl = parser.Template(hot_tpl_empty)
         self.stack = parser.Stack(self.ctx, 'param_id_test', tmpl)
-        self.assertEqual(self.stack.parameters['OS::stack_id'], 'None')
+        self.assertEqual('None', self.stack.parameters['OS::stack_id'])
         self.stack.store()
         stack_identifier = self.stack.identifier()
-        self.assertEqual(self.stack.parameters['OS::stack_id'], self.stack.id)
-        self.assertEqual(self.stack.parameters['OS::stack_id'],
-                         stack_identifier.stack_id)
+        self.assertEqual(self.stack.id, self.stack.parameters['OS::stack_id'])
+        self.assertEqual(stack_identifier.stack_id,
+                         self.stack.parameters['OS::stack_id'])
         self.m.VerifyAll()
 
     def test_set_wrong_param(self):
@@ -787,8 +1003,8 @@ class StackTest(test_parser.StackTest):
         self.stack = parser.Stack(self.ctx, 'update_stack_id_test', tmpl)
         self.stack.store()
         self.stack.create()
-        self.assertEqual(self.stack.state,
-                         (parser.Stack.CREATE, parser.Stack.COMPLETE))
+        self.assertEqual((parser.Stack.CREATE, parser.Stack.COMPLETE),
+                         self.stack.state)
 
         stack_id = self.stack.parameters['OS::stack_id']
 
@@ -800,24 +1016,24 @@ class StackTest(test_parser.StackTest):
         updated_stack = parser.Stack(self.ctx, 'updated_stack', tmpl2)
 
         self.stack.update(updated_stack)
-        self.assertEqual(self.stack.state,
-                         (parser.Stack.UPDATE, parser.Stack.COMPLETE))
-        self.assertEqual(self.stack['AResource'].properties['Foo'], 'xyz')
+        self.assertEqual((parser.Stack.UPDATE, parser.Stack.COMPLETE),
+                         self.stack.state)
+        self.assertEqual('xyz', self.stack['AResource'].properties['Foo'])
 
-        self.assertEqual(
-            self.stack['AResource'].metadata_get()['Bar'], stack_id)
+        self.assertEqual(stack_id,
+                         self.stack['AResource'].metadata_get()['Bar'])
 
     def test_load_param_id(self):
         tmpl = parser.Template(hot_tpl_empty)
         self.stack = parser.Stack(self.ctx, 'param_load_id_test', tmpl)
         self.stack.store()
         stack_identifier = self.stack.identifier()
-        self.assertEqual(self.stack.parameters['OS::stack_id'],
-                         stack_identifier.stack_id)
+        self.assertEqual(stack_identifier.stack_id,
+                         self.stack.parameters['OS::stack_id'])
 
         newstack = parser.Stack.load(self.ctx, stack_id=self.stack.id)
-        self.assertEqual(newstack.parameters['OS::stack_id'],
-                         stack_identifier.stack_id)
+        self.assertEqual(stack_identifier.stack_id,
+                         newstack.parameters['OS::stack_id'])
 
     def test_update_modify_param_ok_replace(self):
         tmpl = {
@@ -837,16 +1053,18 @@ class StackTest(test_parser.StackTest):
                                'update_template_diff')
 
         self.stack = parser.Stack(self.ctx, 'update_test_stack',
-                                  template.Template(tmpl),
-                                  environment.Environment({'foo': 'abc'}))
+                                  template.Template(
+                                      tmpl, env=environment.Environment(
+                                          {'foo': 'abc'})))
         self.stack.store()
         self.stack.create()
         self.assertEqual((parser.Stack.CREATE, parser.Stack.COMPLETE),
                          self.stack.state)
 
         updated_stack = parser.Stack(self.ctx, 'updated_stack',
-                                     template.Template(tmpl),
-                                     environment.Environment({'foo': 'xyz'}))
+                                     template.Template(
+                                         tmpl, env=environment.Environment(
+                                             {'foo': 'xyz'})))
 
         def check_props(*args):
             self.assertEqual('abc', self.stack['AResource'].properties['Foo'])
@@ -855,8 +1073,8 @@ class StackTest(test_parser.StackTest):
             {'Type': 'ResourceWithPropsType',
              'Properties': {'Foo': 'xyz'}},
             {'Type': 'ResourceWithPropsType',
-             'Properties': {'Foo': 'abc'}}).WithSideEffects(check_props) \
-                                           .AndRaise(resource.UpdateReplace)
+             'Properties': {'Foo': 'abc'}}
+        ).WithSideEffects(check_props).AndRaise(resource.UpdateReplace)
         self.m.ReplayAll()
 
         self.stack.update(updated_stack)
@@ -910,7 +1128,7 @@ class StackTest(test_parser.StackTest):
         self.m.VerifyAll()
 
 
-class StackAttributesTest(HeatTestCase):
+class StackAttributesTest(common.HeatTestCase):
     """
     Test stack get_attr function when stack was created from HOT template.
     """
@@ -1007,7 +1225,7 @@ class StackAttributesTest(HeatTestCase):
             self.assertEqual(self.expected, resolved)
 
 
-class StackGetAttrValidationTest(HeatTestCase):
+class StackGetAttrValidationTest(common.HeatTestCase):
 
     def setUp(self):
         super(StackGetAttrValidationTest, self).setUp()
@@ -1036,7 +1254,7 @@ class StackGetAttrValidationTest(HeatTestCase):
                          stack.resources['resource2'].properties['a_string'])
 
 
-class StackParametersTest(HeatTestCase):
+class StackParametersTest(common.HeatTestCase):
     """
     Test stack get_param function when stack was created from HOT template.
     """
@@ -1090,6 +1308,24 @@ class StackParametersTest(HeatTestCase):
                                                 ['none',
                                                  'who_cares']}}},
               expected={'properties': {'prop1': ''}})),
+        ('pseudo_stack_id',
+         dict(params={},
+              snippet={'properties': {'prop1': {'get_param':
+                                                'OS::stack_id'}}},
+              expected={'properties':
+                        {'prop1': '1ba8c334-2297-4312-8c7c-43763a988ced'}})),
+        ('pseudo_stack_name',
+         dict(params={},
+              snippet={'properties': {'prop1': {'get_param':
+                                                'OS::stack_name'}}},
+              expected={'properties': {'prop1': 'test'}})),
+        ('pseudo_project_id',
+         dict(params={},
+              snippet={'properties': {'prop1': {'get_param':
+                                                'OS::project_id'}}},
+              expected={'properties':
+                        {'prop1': '9913ef0a-b8be-4b33-b574-9061441bd373'}})),
+
     ]
 
     props_template = template_format.parse('''
@@ -1117,15 +1353,17 @@ class StackParametersTest(HeatTestCase):
 
     def test_param_refs(self):
         """Test if parameter references work."""
-        tmpl = parser.Template(self.props_template)
         env = environment.Environment(self.params)
-        stack = parser.Stack(utils.dummy_context(), 'test', tmpl, env)
+        tmpl = parser.Template(self.props_template, env=env)
+        stack = parser.Stack(utils.dummy_context(), 'test', tmpl,
+                             stack_id='1ba8c334-2297-4312-8c7c-43763a988ced',
+                             tenant_id='9913ef0a-b8be-4b33-b574-9061441bd373')
         self.assertEqual(self.expected,
                          function.resolve(tmpl.parse(stack, self.snippet)))
 
 
-class HOTParamValidatorTest(HeatTestCase):
-    """Test HOTParamValidator"""
+class HOTParamValidatorTest(common.HeatTestCase):
+    """Test HOTParamValidator."""
 
     def test_multiple_constraint_descriptions(self):
         len_desc = 'string length should be between 8 and 16'
@@ -1302,6 +1540,19 @@ class HOTParamValidatorTest(HeatTestCase):
         value = "0"
         self.assertTrue(v(value))
 
+    def test_custom_constraint_default_skip(self):
+        schema = {
+            'type': 'string',
+            'constraints': [{
+                'custom_constraint': 'skipping',
+                'description': 'Must be skipped on default value'
+            }],
+            'default': 'foo'
+        }
+        param_schema = hot_param.HOTParamSchema.from_dict('p', schema)
+
+        param_schema.validate()
+
     def test_range_constraint_invalid_default(self):
         range_desc = 'Value must be between 30000 and 50000'
         param = {
