@@ -12,19 +12,19 @@
 #    under the License.
 
 import copy
+
 import mock
 import six
 
 from heat.common import exception as heat_ex
 from heat.common import template_format
 from heat.engine.clients.os import nova
-from heat.engine.resources.nova_floatingip import NovaFloatingIp
-from heat.engine.resources.nova_floatingip import NovaFloatingIpAssociation
+from heat.engine.resources.openstack.nova import nova_floatingip
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
+from heat.tests.nova import fakes as fakes_nova
 from heat.tests import utils
-from heat.tests.v1_1 import fakes
 
 
 floating_ip_template = '''
@@ -57,8 +57,7 @@ floating_ip_template_with_assoc = '''
 '''
 
 
-class NovaFloatingIPTest(HeatTestCase):
-
+class NovaFloatingIPTest(common.HeatTestCase):
     def setUp(self):
         super(NovaFloatingIPTest, self).setUp()
 
@@ -70,7 +69,8 @@ class NovaFloatingIPTest(HeatTestCase):
         self.m.StubOutWithMock(self.novaclient.servers, 'get')
         self.m.StubOutWithMock(self.novaclient.servers, 'add_floating_ip')
         self.m.StubOutWithMock(self.novaclient.servers, 'remove_floating_ip')
-        self.stub_keystoneclient()
+        nova.NovaClientPlugin.get_server = mock.Mock(
+            return_value=mock.MagicMock())
 
     def _make_obj(self, **kwargs):
         mock = self.m.CreateMockAnything()
@@ -92,7 +92,9 @@ class NovaFloatingIPTest(HeatTestCase):
         stack = utils.parse_stack(template)
         floating_ip = stack.t.resource_definitions(stack)['MyFloatingIP']
 
-        return NovaFloatingIp('MyFloatingIP', floating_ip, stack)
+        return nova_floatingip.NovaFloatingIp('MyFloatingIP',
+                                              floating_ip,
+                                              stack)
 
     def prepare_floating_ip_assoc(self):
         nova.NovaClientPlugin._create().AndReturn(
@@ -111,8 +113,8 @@ class NovaFloatingIPTest(HeatTestCase):
         resource_defns = stack.t.resource_definitions(stack)
         floating_ip_assoc = resource_defns['MyFloatingIPAssociation']
 
-        return NovaFloatingIpAssociation('MyFloatingIPAssociation',
-                                         floating_ip_assoc, stack)
+        return nova_floatingip.NovaFloatingIpAssociation(
+            'MyFloatingIPAssociation', floating_ip_assoc, stack)
 
     def test_floating_ip_create(self):
         rsrc = self.prepare_floating_ip()
@@ -146,7 +148,7 @@ class NovaFloatingIPTest(HeatTestCase):
     def test_delete_floating_ip_assoc_successful_if_create_failed(self):
         rsrc = self.prepare_floating_ip_assoc()
         self.novaclient.servers.add_floating_ip(None, '11.0.0.1').AndRaise(
-            fakes.fake_exception(400))
+            fakes_nova.fake_exception(400))
 
         self.m.ReplayAll()
 
@@ -170,6 +172,9 @@ class NovaFloatingIPTest(HeatTestCase):
 
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
+        self.assertIsNotNone(rsrc.id)
+        self.assertEqual(rsrc.id, rsrc.resource_id)
+
         self.m.VerifyAll()
 
     def test_floating_ip_assoc_delete(self):
@@ -196,6 +201,38 @@ class NovaFloatingIPTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def create_delete_assoc_with_exc(self, exc_code):
+        rsrc = self.prepare_floating_ip_assoc()
+        self.novaclient.servers.add_floating_ip(None, '11.0.0.1')
+        self.novaclient.servers.get(
+            "67dc62f9-efde-4c8b-94af-013e00f5dc57").AndReturn("server")
+        self.novaclient.floating_ips.get('1').AndReturn(
+            self._make_obj(**{
+                "id": "1",
+                "ip": "11.0.0.1",
+                "pool": "public"
+            })
+        )
+        self.novaclient.servers.remove_floating_ip("server",
+                                                   "11.0.0.1").AndRaise(
+            fakes_nova.fake_exception(exc_code))
+
+        self.m.ReplayAll()
+
+        rsrc.validate()
+        scheduler.TaskRunner(rsrc.create)()
+        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        scheduler.TaskRunner(rsrc.delete)()
+        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
+
+        self.m.VerifyAll()
+
+    def test_floating_ip_assoc_delete_conflict(self):
+        self.create_delete_assoc_with_exc(exc_code=409)
+
+    def test_floating_ip_assoc_delete_not_found(self):
+        self.create_delete_assoc_with_exc(exc_code=404)
+
     def test_floating_ip_assoc_update_server_id(self):
         rsrc = self.prepare_floating_ip_assoc()
         # for create
